import numpy as np
import pylab as pl

from util import goodfigwidth, plshow, plsavefig, txt, axhline, axvline

def kinetic(x, theta, periodic=False):
    N = len(x)
    dx = x[1] - x[0]
    T = np.zeros((N, N), dtype=complex)
    Tflat = T.ravel()
    Tflat[0::N+1] = -2.0
    Tflat[1::N+1] = 1.0
    Tflat[N::N+1] = 1.0
    if periodic:
        T[-1, 0] = 1.0
        T[0, -1] = 1.0
    T *= -0.5 / dx**2 * np.exp(-2.0j * theta)
    return T


def kinetic2(x, theta):
    N = len(x)
    dx = x[1] - x[0]
    T = np.zeros((N, N), dtype=x.dtype)
    X = np.zeros((N, N), dtype=x.dtype)

    L = x[-1]
    P = 2.0 * L / np.pi

    Q = np.zeros_like(T)
    for i in range(N):
        for j in range(N):
            #Q[i, j] = x[i] - x[j]
            Q[i, j] = float(i - j) / N
    Q *= 2.0 * np.pi
    for p in range(1, (N - 1) // 2 + 1):
        T += np.cos(p*Q)*(p/P)**2
    factor = 4.0 / N
    if theta is not None:
        factor *= np.exp(-2j*theta)
    T *= factor
    return T

def main():
    N = 401
    R = 10.0

    theta = 0.4
    x_g = np.linspace(-R, R, N)
    dx = x_g[1] - x_g[0]
    z_g = x_g * np.exp(1j * theta)
    
    energyscale = 0.2

    E0 = 4.0
    vE_g = energyscale * E0 * z_g
    
    Z = 4.0
    softness = 0.15
    
    Vsoft_g = -energyscale * Z / np.sqrt(z_g**2 + softness**2)
    T_gg = energyscale * kinetic(x_g, theta)
    
    v0_g = Vsoft_g + vE_g

    H_gg = T_gg + np.diag(v0_g)
    
    eps_n, psi_gn = np.linalg.eig(H_gg)
    for n, psi_g in enumerate(psi_gn.T):
        norm = np.dot(psi_g, psi_g) * dx
        psi_g /= norm**.5

    args = np.argsort((eps_n * np.exp(1j * (np.pi / 2 - theta))).real)
    eps_n = (eps_n[args])
    psi_gn = (psi_gn[:, args])

    resonances = range(2)
    
    #fig = pl.figure(figsize=(goodfigwidth, 4.5))
    #fig.subplots_adjust(wspace=0.25, hspace=0.25)
    #ax1 = fig.add_subplot(221)
    #ax2 = fig.add_subplot(222)
    #ax3 = fig.add_subplot(212)

    fig1 = pl.figure(figsize=(goodfigwidth / 2.1 - 0.05, 1.8))
    fig2 = pl.figure(figsize=(goodfigwidth / 2.1 + 0.05, 1.8))
    fig3 = pl.figure(figsize=(goodfigwidth * .85, 1.8 * .85))
    fig1.subplots_adjust(right=.97, bottom=.18, left=.27, top=.99)
    fig2.subplots_adjust(right=.93, bottom=.18, top=.74)
    fig3.subplots_adjust(right=.99, bottom=.23, top=.97, left=.2)

    ax1 = fig1.add_subplot(111)
    ax2 = fig2.add_subplot(111)
    ax3 = fig3.add_subplot(111)

    
    arb = r'\ [arb.\ units]'
    ax1.plot(x_g, v0_g.real, 'b-',
             label=r'$\mathrm{Re}\ v^{\theta}(x)$')
    ax1.plot(x_g, v0_g.imag, 'g', dashes=[3,1,1,1],
             label=r'$\mathrm{Im}\ v^{\theta}(x)$')
    ax1.legend(loc='upper left', handletextpad=0.01)
    axhline(ax1)
    #ax1.set_xlabel(r'$x %s$' % arb)
    ax1.set_xlabel(txt(r'$x$ %s' % arb))
    ax1.set_ylabel(txt(r'a) $\, v^\theta(x)$ %s' % arb))
    ax1.axis(xmin=-4, xmax=4, ymax=energyscale * 22, ymin=-energyscale * 30)
    ax1.set_xticks(range(-4, 5, 2))


    ax2.set_xlabel(txt(r'$x$ %s' % arb))
    ax2.set_ylabel(txt(r'b) $\,\psi_\theta(x)$ %s' % arb))

    axhline(ax3)
    axvline(ax3)

    print 'energies', eps_n[resonances]
    ax3.plot(eps_n[resonances].real, eps_n[resonances].imag, 'ro',
             zorder=8,
             label=txt('Desired resonances'))
    ax3.plot(eps_n.real, eps_n.imag, 'bo', zorder=7,
             label=txt('Other states'))

    Z1 = -65 * np.exp(1j * theta)
    Z2 = 65 * np.exp(1j * theta)
    ax3.plot([Z1.real, Z2.real], [Z1.imag, Z2.imag], dashes=(3,2), color='k')
    ax3.axis('equal')
    ax3.set_ylabel(txt(r'c) $\,\mathrm{Im}(\epsilon)$ %s' % arb))
    ax3.set_xlabel(txt(r'$\mathrm{Re}(\epsilon)$ %s' % arb))
    ax3.axis(xmin=-energyscale * 38, xmax=energyscale * 38, 
             ymin=-energyscale * 18, ymax=energyscale * 15)
    ax3.legend(loc='upper left', numpoints=1)
    
    dashes = [[3, 0], [3, 1], [3,1,1,1]]
    for r, res in enumerate(resonances):
        psi_g = psi_gn[:, res]
        ax2.plot(x_g, psi_g.real, color='br'[r], #dashes=dashes[r],
                 label=r'$\mathrm{Re}\ \psi_{\theta %d}(x)$' % (r + 1))
        ax2.plot(x_g, psi_g.imag, color='br'[r], dashes=[3, 1, 1, 1],
                 label=r'$\mathrm{Im}\ \psi_{\theta %d}(x)$' % (r + 1))

    #ax2.legend(loc='lower center', ncol=1, 
    #           handletextpad=0.02, columnspacing=0.6)
    ax2.legend(loc='lower center', bbox_to_anchor=(0.47, 1.003), ncol=2,
               #loc=(-0.03, 1.02), ncol=2,
               handletextpad=0.02, columnspacing=0.6)
    ax2.set_yticks([])
    
    axhline(ax2)
    ax2.axis(xmin=-3.5, xmax=2, ymin=psi_gn[:, 1].min() * 1.1, 
             ymax=psi_gn[:, 0].max() * 1.1)

    for i, fig in enumerate([fig1, fig2, fig3]):
        plsavefig(fig, 'stark-1d.%d' % i)
    plshow()

main()
